import pymc3 as pm
import numpy as np
import matplotlib.pyplot as plt

Data = [# ID  SA  SB
        ['A', 11, 10],
        ['A', 12, 11],
        ['A', 11, 12],
        ['A', 10, 11],
        ['A', 11, 11],
        ['B', 13, 12],
        ['B', 14, 13],
        ['B', 13, 14],
        ['B', 12, 13],
        ['B', 13, 13],
        ['C', 15, 14],
        ['C', 16, 15],
        ['C', 15, 16],
        ['C', 14, 15],
        ['C', 15, 15]
        ]

N = len(Data)
Y = []
X = []
XA = [0] * N
XB = [0] * N
XC = [0] * N
pos = 0
for v in Data:
    Y.append(v[2])
    X.append(v[1])
    if v[0] == 'A':
        XA[pos] = 1
    elif v[0] == 'B':
        XB[pos] = 1
    else:
        XC[pos] = 1
    pos += 1
        
with pm.Model() as MReg3:
	b = pm.Uniform('b', lower = -1000, upper = 1000)
	bA = pm.Uniform('bA', lower = -1000, upper = 1000)
	bB = pm.Uniform('bB', lower = -1000, upper = 1000)
	bC = pm.Uniform('bC', lower = -1000, upper = 1000)
	Y_est = b * X + bA * XA + bB * XB + bC * XC
	sgm = pm.Uniform('sgm', lower = 0, upper = 1000)
	Y_obs = pm.Normal('Y_obs', mu = Y_est, sd = sgm, observed = Y)
	trace = pm.sample()
	pm.traceplot(trace)
	plt.show()

	summary_trace = pm.summary(trace)
	print('\n', summary_trace)

	smpl_b = trace['b']

	bp025 = np.percentile(smpl_b, 2.5)
	bp975 = np.percentile(smpl_b, 97.5)
	plt.hist(smpl_b)
	plt.xlabel('b')
	plt.title('95% CI = [{0:.3f}, {1:.3f}]'.format(bp025, bp975))
	plt.show()

	smpl_bA = trace['bA']
	smpl_bB = trace['bB']
	smpl_bC = trace['bC']
	
	Yregs = []
	for j in range(N):
		Yregs.append([])

	for sb, sbA, sbB, sbC in zip(smpl_b, smpl_bA, smpl_bB, smpl_bC):
		for j in range(N):
			v = sb * X[j] + sbA * XA[j] + sbB * XB[j] + sbC * XC[j]
			Yregs[j].append(v)

	meanYreg = [0] * N
	for i in range(N):
		meanYreg[i] = np.median(Yregs[i])

	plt.plot(meanYreg, Y, 'bo')
	plt.xlabel('Yreg')
	plt.ylabel('Y')
	minYreg = np.amin(meanYreg)
	maxYreg = np.amax(meanYreg)
	plt.plot([minYreg, maxYreg], [minYreg, maxYreg],
		     'b--', label = 'y = x')
	plt.legend()
	plt.show()
	
